@@ -582,6 +582,9 @@ main_event(void)
 	  break;
 	    case ButtonRelease:
     EndRubberBand(canvas, &rubber_data, &event);
+	  break;
+            default: 
+    screenhack_handle_event (dpy, &event);
 	  break;
 	    }
 	}
